// --------------------------------------------------------------------
// >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
// --------------------------------------------------------------------
// Module:Thermometer 
// 
// Author: Step
// 
// Description: Thermometer
// 
// Web: www.stepfpga.com
// 
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Version: |Mod. Date:   |Changes Made:
// V1.0     |2015/11/11   |Initial ver
// --------------------------------------------------------------------
module Thermometer
(
input					clk,		// system clock
input					rst_n,		// system reset, active low
inout					one_wire,	// ds18b20z one-wire-bus

output	        [15:0]	data_out
);

    
    //Drive DS18B20Z to get temperature code
    DS18B20Z DS18B20Z_uut
    (
    .clk					(clk			),	// system clock
    .rst_n					(rst_n			),	// system reset, active low
    .one_wire				(one_wire		),	// ds18b20z one-wire-bus
    .data_out				(data_out		)	// ds18b20z data_out
    );

    // judge sign of temperature
    wire temperature_flag = data_out[15:11]? 1'b0:1'b1;
    // complement if negative
    wire [10:0] temperature_code = temperature_flag? data_out[10:0]:(~data_out[10:0])+1'b1; 
    // translate temperature_code to real temperature
    wire [20:0] bin_code = temperature_code * 16'd625;



endmodule

