

// 为了让串口播放音乐长一点，这边设为了30秒。改成10即符合要求。
`define PLAY_DURATION 30


module Top
#(
    parameter simPresent = 0
)(

    input           CLK,
    input           RSTn,
    input   [3:0]   KEY,
    input   [2:0]   SW,
    
    output  [7:0]   LED,
    output          BEEP,
    
    input           RXD,
    output          TXD,
    
    output			oled_csn,	//OLCD液晶屏使能
    output			oled_rst,	//OLCD液晶屏复位
    output			oled_dcn,	//OLCD数据指令控制
    output			oled_clk,	//OLCD时钟信号
    output			oled_dat,	//OLCD数据信号

    inout			one_wire

);
    
    // uart
    wire    [7:0]   rx_data;
    wire            rx_valid;
    reg     [7:0]   tx_data;
    reg             tx_flag;
    wire            tx_done;
    reg     [4:0]   tx_cnt;
    reg     [7:0]   tx_data_shift[0:15];
    
    // alarm clock 
    wire            pulse;
    wire    [5:0]   second;
    wire    [5:0]   minute;
    wire    [5:0]   hour;
    reg             alert;
    reg             play_valid;
    reg             play_busy;
    reg     [7:0]   play_cnt;
    
    // temperature
    wire    [15:0]  temperature_data;
    wire temperature_flag = temperature_data[15:11]? 1'b0:1'b1;
    wire [10:0] temperature_code = temperature_flag? temperature_data[10:0]:(~temperature_data[10:0])+1'b1; 
    wire [7:0] temperature = temperature_code[10:3]>>1;
    
    // encode
    wire [7:0]      temp0 = temperature / 10;
    wire [7:0]      temp1 = temperature % 10;
    wire [7:0]      time0 = hour / 10;
    wire [7:0]      time1 = hour %10;
    wire [7:0]      time2 = minute / 10;
    wire [7:0]      time3 = minute % 10;
    reg [15:0]      oled1;                                                                                                                                  
    reg [15:0]      oled2;
    reg [15:0]      oled3;
    always @(posedge CLK) if(!play_busy) oled1 <= {temp0, temp1}; 
    always @(posedge CLK) if(!play_busy) oled2 <= {time0, time1}; 
    always @(posedge CLK) if(!play_busy) oled3 <= {time2, time3};
    
    always @(posedge CLK, negedge RSTn) begin
        if(!RSTn) alert <= 0;
        else if((minute==0)&(second==30)&pulse) alert <= 0;
        else if((minute==59)&(second==59)&pulse) alert <= 1;
//        else if((minute==30)&pulse) alert <= 0;
//        else if((minute==59)&pulse & SW[0]) alert <= 1;
    end


    Thermometer u_thermometer
    (
        .clk            (CLK        ),		// system clock
        .rst_n          (RSTn       ),		// system reset, active low
        .one_wire       (one_wire   ),	// ds18b20z one-wire-bus
        .data_out       (temperature_data)
    );


    Uart_Bus #(
        .BPS_PARA(1250) 	//12MHz时钟时参数1250对应9600的波特率
    )
    u_uart
    (
        .clk            (CLK        ),			//系统时钟 12MHz
        .rst_n          (RSTn       ),//系统复位，低有效

        .uart_rx        (RXD        ),		//UART接收输入
        .rx_data_valid  (rx_valid   ),	//接收数据有效脉冲
        .rx_data_out    (rx_data    ),	//接收到的数据

        .tx_done        (tx_done    ),
        .tx_data_valid  (tx_flag    ),	//发送数据有效脉冲
        .tx_data_in     (tx_data    ),		//要发送的数据
        .uart_tx        (TXD        )			//UART发送输出
    );	
    
    
    Beeper #(.simPresent(simPresent)) u_beeper
    (
        .clk            (CLK        ),
        .rst_n          (RSTn       ),
        .en             (alert      ),
        .cycle_in       (rx_data    ),
        .pc_play        (play_busy  ),
        .beeper         (BEEP       )
    );
   
    OLED12832 u_oled
    (
        .clk            (CLK        ),
        .rst_n          (RSTn       ),
        
        .oled1          (oled1      ),		//
        .oled2          (oled2      ),
        .oled3          (oled3      ),
        
        .oled_csn       (oled_csn   ),	//OLCD液晶屏使能
        .oled_rst       (oled_rst   ),	//OLCD液晶屏复位
        .oled_dcn       (oled_dcn   ),	//OLCD数据指令控制
        .oled_clk       (oled_clk   ),	//OLCD时钟信号
        .oled_dat       (oled_dat   )	//OLCD数据信号
    );
    
    wire [3:0] key_debounce;
    wire time_enter, time_aug, time_minus, time_set;
    assign {time_enter, time_aug, time_minus, time_set} = key_debounce;
    
    debounce #(.simPresent(simPresent)) u_deb
    (
        .clk            (CLK        ),
        .rst_n          (RSTn       ),
        .key_n          (KEY        ),
        .key_pulse      (key_debounce)
    ); 

    
    alarm #(.simPresent(simPresent)) u_alarm
    (
        .clk            (CLK        ),
        .rst_n          (RSTn       ),
        .pulse          (pulse      ),
        .second         (second     ),
        .hour_o         (hour       ),
        .minute_o       (minute     ),
        .time_enter     (time_enter ),
        .time_aug       (time_aug   ),
        .time_minus     (time_minus ),
        .time_set       (time_set   )
    );
    

    always@(posedge CLK) begin
        tx_data_shift[0] <= " ";
        tx_data_shift[1] <= "-";
        tx_data_shift[2] <= " ";
        tx_data_shift[3] <= " ";
        tx_data_shift[4] <= ":";
        tx_data_shift[5] <= " ";
        tx_data_shift[6] <= " ";
        tx_data_shift[7] <= "-";
        tx_data_shift[8] <= ",";
        tx_data_shift[9] <= " ";
        tx_data_shift[10] <= "T";
        tx_data_shift[11] <= ":";
        tx_data_shift[12] <= " ";
        tx_data_shift[13] <= " ";
        tx_data_shift[14] <= 13;
        tx_data_shift[15] <= "\n";
    end
   
    always @(posedge CLK, negedge RSTn) begin
        if(!RSTn) tx_cnt <= 0;
        else if(tx_cnt==16) tx_cnt <= 0;
        else if(tx_flag&(tx_cnt==0)) tx_cnt <= 1;
        else if((tx_done)&(tx_cnt!=0)) tx_cnt <= tx_cnt+1;
    end
    
    always @(posedge CLK, negedge RSTn) begin
        if(!RSTn) tx_data <= 0;
        else if(tx_cnt==2) tx_data <= time0 + 48;
        else if(tx_cnt==3) tx_data <= time1 + 48;
        else if(tx_cnt==5) tx_data <= time2 + 48;
        else if(tx_cnt==6) tx_data <= time3 + 48;
        else if(tx_cnt==12) tx_data <= temp0 + 48;
        else if(tx_cnt==13) tx_data <= temp1 + 48;
        else if(tx_cnt<=15) tx_data <= tx_data_shift[tx_cnt];
    end
    
    always @(posedge CLK, negedge RSTn) begin
        if(!RSTn) tx_flag <= 0;
        else if(((minute==59)&(second==59)&pulse)) tx_flag <= 1;
        else if((tx_done)&(tx_cnt!=0)) tx_flag <= 1;
        else if(tx_cnt==0) tx_flag <= 0;
        else tx_flag <= 0;
    end
    
    always @(posedge CLK, negedge RSTn) begin
        if(!RSTn) play_valid <= 0;
        else if(alert) play_valid <= 1;
        else if(play_cnt==`PLAY_DURATION) play_valid <= 0;
    end   
    
    always @(posedge CLK, negedge RSTn) begin
        if(!RSTn) play_busy <= 0;
        else if(play_cnt==`PLAY_DURATION) play_busy <= 0;
        else if(rx_valid & play_valid) play_busy <= 1;
    end    
    
    always @(posedge CLK, negedge RSTn) begin
        if(!RSTn) play_cnt <= 0;
        else if(play_cnt==`PLAY_DURATION) play_cnt <= 0;
        else if(pulse & play_busy) play_cnt <= play_cnt + 1;
    end
    
    
    assign LED = ~rx_data;
    
endmodule


