







module alarm
#(

    parameter simPresent = 0

)
(

    input                   clk,
    input                   rst_n,
    
    output  reg             pulse,
    output  reg [5:0]       second,
    output  [5:0]           hour_o,
    output  [5:0]           minute_o,
    
    
    input                   time_enter,
    input                   time_aug,
    input                   time_minus,
    input                   time_set

);


    reg [5:0]       hour;
    reg [5:0]       minute;

    reg [23:0]      cnt;
    reg [5:0]       minute_set;
    reg [5:0]       hour_set;
    reg [3:0]       state;
    reg [3:0]       state_next;
    
    
    localparam ST_IDLE = 0;
    localparam ST_SETHOURS = 1;
    localparam ST_SETMINUTES = 2;
    
    localparam prescale = simPresent ? 12 : 12000000;
    //localparam prescale = simPresent ? 12 : 200000;
    
    always @(posedge clk, negedge rst_n) begin
        if(!rst_n) cnt <= 0;
        else if(cnt==(prescale-1)) cnt<=0;
        else cnt <= cnt+1;
    end

    always @(posedge clk, negedge rst_n) begin
        if(!rst_n) pulse <= 0;
        else pulse <= (cnt==(prescale-1));
    end
    
    always @(posedge clk, negedge rst_n) begin
        if(!rst_n) second <= 0;
        else if(state!=ST_IDLE) second <= 0;
        else if((second==59)&pulse) second <= 0;
        else if(pulse) second <= second + 1;    
    end
    
    always @(posedge clk, negedge rst_n) begin
        if(!rst_n) minute_set <= 0;
        else if((state==ST_IDLE)&time_enter) minute_set <= minute;
        else if((state==ST_SETMINUTES)&time_aug&(minute_set==59)) minute_set <= 0;
        else if((state==ST_SETMINUTES)&time_aug) minute_set <= minute_set + 1;
        else if((state==ST_SETMINUTES)&time_minus&(minute_set==0)) minute_set <= 59;
        else if((state==ST_SETMINUTES)&time_minus) minute_set <= minute_set - 1;  
    end    
    
    always @(posedge clk, negedge rst_n) begin
        if(!rst_n) minute <= 0;
        else if((state!=ST_IDLE)&time_set) minute <= minute_set;
        else if((minute==59)&(second==59)&pulse) minute <= 0;  
        else if((second==59)&pulse) minute <= minute + 1;    
    end

    always @(posedge clk, negedge rst_n) begin
        if(!rst_n) hour_set <= 0;
        else if((state==ST_IDLE)&time_enter) hour_set <= hour;
        else if((state==ST_SETHOURS)&time_aug&(hour_set==23)) hour_set <= 0;
        else if((state==ST_SETHOURS)&time_aug) hour_set <= hour_set + 1;
        else if((state==ST_SETHOURS)&time_minus&(hour_set==0)) hour_set <= 23; 
        else if((state==ST_SETHOURS)&time_minus) hour_set <= hour_set - 1;  
    end
    
    always @(posedge clk, negedge rst_n) begin
        if(!rst_n) hour <= 0;
        else if((state!=ST_IDLE)&time_set) hour <= hour_set;
        else if((hour==23)&(second==59)&(minute==59)&pulse) hour <= 0;  
        else if((second==59)&(minute==59)&pulse) hour <= hour + 1;    
    end
    
    always @(posedge clk, negedge rst_n) begin
        if(!rst_n) state <= ST_IDLE;
        else state <= state_next;
    end


    always @(*) begin
        case(state)
            ST_IDLE: begin if(time_enter) state_next = ST_SETHOURS; else state_next = state; end
            ST_SETHOURS: begin if(time_enter) state_next = ST_SETMINUTES; else if(time_set) state_next = ST_IDLE; else state_next = state; end
            ST_SETMINUTES: begin if(time_enter) state_next = ST_IDLE; else if(time_set) state_next = ST_IDLE; else state_next = state; end
            default: state_next = ST_IDLE;
        endcase
    end
    
    assign hour_o = (state==ST_IDLE) ? hour : hour_set;
    assign minute_o = (state==ST_IDLE) ? minute : minute_set;


endmodule


